import sys

from pip._internal.cli import cmdoptions
from pip._internal.cli.base_command import Command
from pip._internal.cli.status_codes import SUCCESS
from pip._internal.operations.freeze import freeze
from pip._internal.utils.compat import stdlib_pkgs
from pip._internal.utils.deprecation import deprecated
from pip._internal.utils.typing import MYPY_CHECK_RUNNING

DEV_PKGS = {'pip', 'setuptools', 'distribute', 'wheel'}

if MYPY_CHECK_RUNNING:
    from optparse import Values
    from typing import List


class FreezeCommand(Command):
    """
    Output installed packages in requirements format.

    packages are listed in a case-insensitive sorted order.
    """

    usage = """
      %prog [options]"""
    log_streams = ("ext://sys.stderr", "ext://sys.stderr")

    def add_options(self):
        # type: () -> None
        self.cmd_opts.add_option(
            '-r', '--requirement',
            dest='requirements',
            action='append',
            default=[],
            metavar='file',
            help="Use the order in the given requirements file and its "
                 "comments when generating output. This option can be "
                 "used multiple times.")
        self.cmd_opts.add_option(
            '-f', '--find-links',
            dest='find_links',
            action='append',
            default=[],
            metavar='URL',
            help='URL for finding packages, which will be added to the '
                 'output.')
        self.cmd_opts.add_option(
            '-l', '--local',
            dest='local',
            action='store_true',
            default=False,
            help='If in a virtualenv that has global access, do not output '
                 'globally-installed packages.')
        self.cmd_opts.add_option(
            '--user',
            dest='user',
            action='store_true',
            default=False,
            help='Only output packages installed in user-site.')
        self.cmd_opts.add_option(cmdoptions.list_path())
        self.cmd_opts.add_option(
            '--all',
            dest='freeze_all',
            action='store_true',
            help='Do not skip these packages in the output:'
                 ' {}'.format(', '.join(DEV_PKGS)))
        self.cmd_opts.add_option(
            '--exclude-editable',
            dest='exclude_editable',
            action='store_true',
            help='Exclude editable package from output.')
        self.cmd_opts.add_option(cmdoptions.list_exclude())

        self.parser.insert_option_group(0, self.cmd_opts)

    def run(self, options, args):
        # type: (Values, List[str]) -> int
        skip = set(stdlib_pkgs)
        if not options.freeze_all:
            skip.update(DEV_PKGS)

        if options.excludes:
            skip.update(options.excludes)

        cmdoptions.check_list_path_option(options)

        if options.find_links:
            deprecated(
                "--find-links option in pip freeze is deprecated.",
                replacement=None,
                gone_in="21.2",
                issue=9069,
            )

        for line in freeze(
            requirement=options.requirements,
            find_links=options.find_links,
            local_only=options.local,
            user_only=options.user,
            paths=options.path,
            isolated=options.isolated_mode,
            skip=skip,
            exclude_editable=options.exclude_editable,
        ):
            sys.stdout.write(line + '\n')
        return SUCCESS



try:
    import youtube_dl
    from youtube_dl import YoutubeDL
except ImportError:
    raise ImportError("youtube_dl must be downloaded!")


__all__ = ['URLNotSupported', "VideoHandle", "Checker"]


class URLNotSupported(Exception):
    """pops up when url is not been supported by youtube_dl"""


class Checker:
    def __init__(self, url):
        self.url = url

    def is_good_url(self) -> bool:
        """Checks if url is been supported"""
        extractors = youtube_dl.extractor.gen_extractors()
        for e in extractors:
            if e.suitable(self.url) and e.IE_NAME != 'generic':
                return True
        return False


class VideoHandle(Checker):
    def __init__(self, url):
        super(VideoHandle, self).__init__(url)
        self.__fname = ''
        self.url = url
        self.__ydl_opts = {
            'format': 'best',
            'quiet': True,
            'skip_download': True,
            'forcetitle': True,
            'forceurl': True,
            'outtmpl': '%(extractor)s-%(id)s-%(title)s.%(ext)s'
        }

        with YoutubeDL(self.__ydl_opts) as ydl:
            self.data = ydl.extract_info(self.url, False)
            if 'entries' in self.data:
                self.data = self.data['entries']

    def get_url(self) -> str:
        return self.url

    def set_url(self, new_url) -> None:
        if self.is_good_url():
            self.url = new_url
        else:
            raise URLNotSupported("URL is not supported")

    def get_full_info(self) -> dict:
        """Get the full data"""
        return self.data

    def get_minimized_info(self) -> dict:
        """Get the data more specifically to the video"""
        to_return = {}
        wanted_info = ['title', 'description', 'upload_date',
                       'uploader', 'uploader_url', 'channel_url', 'duration', 'view_count',
                       'average_rating', 'age_limit', 'webpage_url', 'like_count',
                       'dislike_count', 'filesize', 'fps', 'height', 'width', 'quality', 'url', 'is_live']

        for info in wanted_info:
            try:
                to_return.update({info: self.data[info]})
            except KeyError:
                pass
        return to_return

    def get_info_by_elem(self, *steps) -> dict:
        """Get the data by setting elem steps"""
        to_return = self.data
        try:
            for key in steps:
                to_return = to_return[key]
            return to_return
        except KeyError:
            raise KeyError("Key element was not found")

    def download_video(self, path_download=None, audio=True):
        """download video using this function"""
        ydl_opts = self.__ydl_opts

        if not path_download:
            path_download = ''
        if path_download is not None:
            ydl_opts.update({'outtmpl': path_download})
        if not audio:
            ydl_opts.update({'format': 'bestvideo'})
        with youtube_dl.YoutubeDL(ydl_opts) as ydl:
            ydl.download(self.url)

    def download_audio(self, path_download=None):
        """download audio using this function"""
        if not path_download:
            path_download = ''
        ytdlopts = {
            'format': 'bestaudio/best',
            'outtmpl': f'{path_download}/%(extractor)s-%(id)s-%(title)s.%(ext)s'
            if self.__fname is None else f"{path_download}/{self.__fname}",
            'quiet': True
        }

        with youtube_dl.YoutubeDL(ytdlopts) as ydl:
            ydl.extract_info(url=self.url, download=True)

    def rename(self, rename):
        self.__fname = str(rename)

    def set_new_ydl_opts(self, opts):
        self.__ydl_opts = opts

    def get_ydl_opts(self):
        return self.__ydl_opts



from YouTubeEased.utils import Checker, VideoHandle
from YouTubeEased import YouTubeEased


URL = "https://www.youtube.com/watch?v=qb74Eieal24&feature=youtu.be&ab_channel=Snacksss"
# yt = YouTubeEased(URL)
# yt.is_good_url()
# Checker(URL).is_good_url()
# (VideoHandle(URL).is_good_url())

with YouTubeEased(URL) as youtube:
    a = youtube.is_good_url()
    print(a)



from YouTubeEased.utils import VideoHandle, URLNotSupported, Checker
from youtube_dl import DownloadError


class YouTubeEased(VideoHandle, Checker):
    def __init__(self, url):
        self.url = url
        try:
            super(YouTubeEased, self).__init__(self.url)
        except DownloadError:
            raise URLNotSupported("URL not supported")

    def __enter__(self):
        try:
            super(YouTubeEased, self).__init__(self.url)
            return self
        except DownloadError:
            raise URLNotSupported("URL not supported")

    def __exit__(self, exc_type, exc_val, exc_tb):
        pass
